precision highp float;

varying vec2 vTextureCoord;
uniform sampler2D sTexture;

uniform float intensity;
uniform float imageWidth;
uniform float imageHeight;

vec4 doSharp(vec4 curColor) {
    if (intensity != 0.0) {
        float bitmapMaxLength = max(imageHeight, imageWidth);
        float f = abs(intensity);

        float f2 = (bitmapMaxLength - 1000.0) / 2000.0;
        f2 = f2 > 1.0 ? 1.0 : (f2 < 0.0 ? 0.0 : f2);
        f = ((f * 4.0) * (((1.0 - f2) * 0.65) + (f2 * 1.2))) + 1.0;
        float f3 = (1.0 - f) * 0.25;

        vec4 color_left = texture2D(sTexture, vTextureCoord+vec2(-1.0/imageWidth, 0.0));
        vec4 color_right = texture2D(sTexture, vTextureCoord+vec2(1.0/imageWidth, 0.0));
        vec4 color_bottom = texture2D(sTexture, vTextureCoord+vec2(0.0, 1.0/imageHeight));
        vec4 color_top = texture2D(sTexture, vTextureCoord+vec2(0.0, -1.0/imageHeight));

        vec4 newColor = vec4(0.0, 0.0, 0.0, curColor.a);
        newColor.rgb = f*curColor.rgb + f3*color_left.rgb + f3*color_right.rgb + f3*color_top.rgb + f3*color_bottom.rgb;

        curColor = newColor;
    }
    return curColor;
}

void main()
{
    vec4 curColor = texture2D(sTexture, vTextureCoord);

    curColor = doSharp(curColor);

    gl_FragColor = curColor;
}
