precision mediump float;
varying vec2 vTextureCoord;

uniform sampler2D sTexture;
uniform sampler2D uTexture_1;

uniform float uProgress;// result:(0 ~ 1)

const vec4 shadow_colour = vec4(0.,0.,0.,.6);
const float shadow_height  = 0.075;
const float bounces = 3.0;

const float PI = 3.14159265358;

vec4 transition (vec2 uv);

void main() {
    gl_FragColor = transition(vTextureCoord);
}

vec4 getFromColor(in vec2 uv){
    return texture2D(sTexture, uv);
}

vec4 getToColor(in vec2 uv){
    return texture2D(uTexture_1, uv);
}

vec4 transition (vec2 uv) {
    float time = uProgress;
    float stime = sin(time * PI / 2.);
    float phase = time * PI * bounces;
    float y = (abs(cos(phase))) * (1.0 - stime);
    float d = uv.y - y;
    return mix(
               mix(
                   getToColor(uv),
                   shadow_colour,
                   step(d, shadow_height) * (1. - mix(
                                                      ((d / shadow_height) * shadow_colour.a) + (1.0 - shadow_colour.a),
                                                      1.0,
                                                      smoothstep(0.95, 1., uProgress) // fade-out the shadow at the end
                                                      ))
                   ),
               getFromColor(vec2(uv.x, uv.y + (1.0 - y))),
               step(d, 0.0)
               );
}

