precision mediump float;
varying vec2 vTextureCoord;

uniform sampler2D sTexture;
uniform sampler2D uTexture_1;

uniform float uProgress;// result:(0 ~ 1)

const float amplitude = 1.0; // = 1.0
const float waves = 30.0; // = 30.0
const float colorSeparation = 0.3; // = 0.3
const float PI = 3.14159265358979323846264;

vec4 transition (vec2 uv);

void main() {
    gl_FragColor = transition(vTextureCoord);
}

vec4 getFromColor(in vec2 uv){
    return texture2D(sTexture, uv);
}

vec4 getToColor(in vec2 uv){
    return texture2D(uTexture_1, uv);
}

float compute(vec2 p, float progress, vec2 center) {
    vec2 o = p*sin(progress * amplitude)-center;
    // horizontal vector
    vec2 h = vec2(1., 0.);
    // butterfly polar function (don't ask me why this one :))
    float theta = acos(dot(o, h)) * waves;
    return (exp(cos(theta)) - 2.*cos(4.*theta) + pow(sin((2.*theta - PI) / 24.), 5.)) / 10.;
}
vec4 transition(vec2 uv) {
    vec2 p = uv.xy / vec2(1.0).xy;
    float inv = 1. - uProgress;
    vec2 dir = p - vec2(.5);
    float dist = length(dir);
    float disp = compute(p, uProgress, vec2(0.5, 0.5)) ;
    vec4 texTo = getToColor(p + inv*disp);
    vec4 texFrom = vec4(
                        getFromColor(p + uProgress*disp*(1.0 - colorSeparation)).r,
                        getFromColor(p + uProgress*disp).g,
                        getFromColor(p + uProgress*disp*(1.0 + colorSeparation)).b,
                        1.0);
    return texTo*uProgress + texFrom*inv;
}

