precision mediump float;
varying vec2 vTextureCoord;

uniform sampler2D sTexture;
uniform sampler2D uTexture_1;

uniform float uProgress;// result:(0 ~ 1)

#define DEG2RAD 0.03926990816987241548078304229099 // 1/180*PI


// Transition parameters --------

// In degrees
const float rotation = 6.0; // = 6

// Multiplier
const float scale = 1.2;

vec4 transition (vec2 uv);

void main() {
    gl_FragColor = transition(vTextureCoord);
}

vec4 getFromColor(in vec2 uv){
    return texture2D(sTexture, uv);
}

vec4 getToColor(in vec2 uv){
    return texture2D(uTexture_1, uv);
}

vec4 transition(vec2 uv) {
    // Massage parameters
    float phase = uProgress < 0.5 ? uProgress * 2.0 : (uProgress - 0.5) * 2.0;
    float angleOffset = uProgress < 0.5 ? mix(0.0, rotation * DEG2RAD, phase) : mix(-rotation * DEG2RAD, 0.0, phase);
    float newScale = uProgress < 0.5 ? mix(1.0, scale, phase) : mix(scale, 1.0, phase);
    
    vec2 center = vec2(0, 0);
    float ratio = 1.0;

    // Calculate the source point
    vec2 assumedCenter = vec2(0.5, 0.5);
    vec2 p = (uv.xy - vec2(0.5, 0.5)) / newScale * vec2(ratio, 1.0);
    // This can probably be optimized (with distance())
    float angle = atan(p.y, p.x) + angleOffset;
    float dist = distance(center, p);
    p.x = cos(angle) * dist / ratio + 0.5;
    p.y = sin(angle) * dist + 0.5;
    vec4 c = uProgress < 0.5 ? getFromColor(p) : getToColor(p);
    
    // Finally, apply the color
    return c + (uProgress < 0.5 ? mix(0.0, 1.0, phase) : mix(1.0, 0.0, phase));
}

