precision mediump float;
varying vec2 vTextureCoord;

uniform sampler2D sTexture;
uniform sampler2D uTexture_1;

uniform float uProgress;// result:(0 ~ 1)

const float reflection = 0.4;
const float perspective = 0.2;
const float depth = 3.0;

const vec4 black = vec4(0.0, 0.0, 0.0, 1.0);
const vec2 boundMin = vec2(0.0, 0.0);
const vec2 boundMax = vec2(1.0, 1.0);

vec4 transition (vec2 uv);

void main() {
    gl_FragColor = transition(vTextureCoord);
}

vec4 getFromColor(in vec2 uv){
    return texture2D(sTexture, uv);
}

vec4 getToColor(in vec2 uv){
    return texture2D(uTexture_1, uv);
}

bool inBounds (vec2 p) {
    return all(lessThan(boundMin, p)) && all(lessThan(p, boundMax));
}

vec2 project (vec2 p) {
    return p * vec2(1.0, -1.2) + vec2(0.0, -0.02);
}

vec4 bgColor (vec2 p, vec2 pfr, vec2 pto) {
    vec4 c = black;
    pfr = project(pfr);
    if (inBounds(pfr)) {
        c += mix(black, getFromColor(pfr), reflection * mix(1.0, 0.0, pfr.y));
    }
    pto = project(pto);
    if (inBounds(pto)) {
        c += mix(black, getToColor(pto), reflection * mix(1.0, 0.0, pto.y));
    }
    return c;
}

vec4 transition(vec2 p) {
    vec2 pfr, pto = vec2(-1.);
    
    float size = mix(1.0, depth, uProgress);
    float persp = perspective * uProgress;
    pfr = (p + vec2(-0.0, -0.5)) * vec2(size/(1.0-perspective*uProgress), size/(1.0-size*persp*p.x)) + vec2(0.0, 0.5);
    
    size = mix(1.0, depth, 1.-uProgress);
    persp = perspective * (1.-uProgress);
    pto = (p + vec2(-1.0, -0.5)) * vec2(size/(1.0-perspective*(1.0-uProgress)), size/(1.0-size*persp*(0.5-p.x))) + vec2(1.0, 0.5);
    
    if (uProgress < 0.5) {
        if (inBounds(pfr)) {
            return getFromColor(pfr);
        }
        if (inBounds(pto)) {
            return getToColor(pto);
        }
    }
    if (inBounds(pto)) {
        return getToColor(pto);
    }
    if (inBounds(pfr)) {
        return getFromColor(pfr);
    }
    return bgColor(p, pfr, pto);
}

